#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gemtext-parser.h"

gemtextNodeQueue lq;
gemtextParser    parser;

char * preBlk =
    "Just a regular preformatted block.\n"
    "Nothing special";

int main() {
    int          ret = 0;
    FILE        *stream = NULL;
    gemtextNode *node = NULL;

    stream = fopen("test1.gmi", "r");
    assert(stream != NULL);
    ret = gemtextNodeQueueInit(&lq);
    assert(ret == 0);
    ret = gemtextParserInit(&parser, stream);
    assert(ret == 0);
    ret = parseGemtext(&parser, &lq);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType = h2Node);
    assert(memcmp(node->str, "A more complicated example", 26) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == listNode);
    assert(memcmp(node->str, "list item with no leading space", 30) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == listNode);
    assert(memcmp(node->str, "list item with several leading spaces", 37) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == h2Node);
    assert(memcmp(node->str, "After this H2, an empty quote", 29) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == normalNode);
    assert(memcmp(node->str, "\n", 1) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == h3Node);
    assert(memcmp(node->str, "Now we'll", 9) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == normalNode);
    assert(memcmp(node->str, "``", 2) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == normalNode);
    assert(memcmp(node->str, "=", 1) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == normalNode);
    assert(memcmp(node->str, "And maybe", 9) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == linkNode);
    assert(memcmp(node->link->url, "spartan://example.org", 21) == 0);
    assert(node->link->display == NULL);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == normalNode);
    assert(memcmp(node->str, "Let's enter", 11) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == preformattedNode);
    assert(node->block->altText == NULL);
    assert(memcmp(node->block->body, preBlk, 50) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == normalNode);
    assert(memcmp(node->str, "And we'll finish", 16) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node->nodeType == linkNode);
    assert(node->link->display == NULL);
    assert(memcmp(node->link->url, "finger://example.org/joe", 24) == 0);
    gemtextNodeDeinit(node);

    node = gemtextNodeQueueTryPop(&lq);
    assert(node != NULL);
    assert(node->nodeType == endOfStream);
    gemtextNodeDeinit(node);

     return ret;
}
